package io.gitlab.jfronny.libjf;

import net.fabricmc.loader.api.FabricLoader;
import net.fabricmc.loader.api.ModContainer;
import net.fabricmc.loader.api.metadata.CustomValue;
import org.jetbrains.annotations.ApiStatus;

import java.util.LinkedHashSet;
import java.util.Set;

@ApiStatus.Internal
public class Flags {
    public static Set<StringFlag> getStringFlags(String name) {
        Set<StringFlag> flags = new LinkedHashSet<>();
        String propName = LibJf.MOD_ID + "." + name;
        if (System.getProperty(propName) != null)
            flags.add(new StringFlag("System Property", System.getProperty(propName)));
        for (ModContainer mod : FabricLoader.getInstance().getAllMods()) {
            if (!mod.getMetadata().containsCustomValue(LibJf.MOD_ID)) continue;
            CustomValue.CvObject co = mod.getMetadata().getCustomValue(LibJf.MOD_ID).getAsObject();
            if (!co.containsKey(name)) continue;
            flags.add(new StringFlag("Mod: " + mod.getMetadata().getId(), co.get(name).getAsString()));
        }
        return flags;
    }

    public static Set<BooleanFlag> getBoolFlags(String name) {
        Set<BooleanFlag> flags = new LinkedHashSet<>();
        String propName = LibJf.MOD_ID + "." + name;
        if (System.getProperty(propName) != null)
            flags.add(new BooleanFlag("System Property", true));
        for (ModContainer mod : FabricLoader.getInstance().getAllMods()) {
            if (!mod.getMetadata().containsCustomValue(LibJf.MOD_ID)) continue;
            CustomValue.CvObject co = mod.getMetadata().getCustomValue(LibJf.MOD_ID).getAsObject();
            if (!co.containsKey(name)) continue;
            flags.add(new BooleanFlag("Mod: " + mod.getMetadata().getId(), co.get(name).getAsBoolean()));
        }
        return flags;
    }

    public record StringFlag(String source, String value) {
    }

    public record BooleanFlag(String source, boolean value) {
    }
}
