package io.gitlab.jfronny.libjf.config.impl;

import io.gitlab.jfronny.libjf.LibJf;
import io.gitlab.jfronny.libjf.config.api.Entry;
import net.fabricmc.loader.api.FabricLoader;

import java.lang.reflect.Field;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

/** Based on https://github.com/TeamMidnightDust/MidnightLib which is based on https://github.com/Minenash/TinyConfig
 * Credits to TeamMidnightDust and Minenash */

public class Config {
    public final List<EntryInfo> entries = new ArrayList<>();
    public Path path;
    public final String modid;
    public final Class<?> configClass;

    public Config(String modid, Class<?> config) {
        this.modid = modid;
        configClass = config;
        path = FabricLoader.getInstance().getConfigDir().resolve(modid + ".json");

        for (Field field : config.getFields()) {
            EntryInfo info = new EntryInfo();
            info.field = field;
            if (field.isAnnotationPresent(Entry.class))
                try {
                    info.defaultValue = field.get(null);
                } catch (IllegalAccessException ignored) {}
            entries.add(info);
        }
        try {
            LibJf.GSON.fromJson(Files.newBufferedReader(path), config); }
        catch (Exception e) { write(); }
    }

    public void write() {
        path = FabricLoader.getInstance().getConfigDir().resolve(modid + ".json");
        try {
            if (!Files.exists(path)) Files.createFile(path);
            Files.write(path, LibJf.GSON.toJson(configClass.getDeclaredConstructor().newInstance()).getBytes());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
