package io.gitlab.jfronny.libjf.data.manipulation.api;

import io.gitlab.jfronny.libjf.data.manipulation.impl.ResourcePackHook;
import io.gitlab.jfronny.libjf.interfaces.ThrowingRunnable;
import io.gitlab.jfronny.libjf.interfaces.ThrowingSupplier;
import net.fabricmc.fabric.api.event.Event;
import net.fabricmc.fabric.api.event.EventFactory;
import net.minecraft.resource.ResourcePack;
import net.minecraft.resource.ResourceType;
import net.minecraft.util.Identifier;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.function.Predicate;

public class UserResourceEvents {
    public static <TVal, TEx extends Throwable> TVal disable(ThrowingSupplier<TVal, TEx> then) throws TEx {
        try {
            ResourcePackHook.setDisabled(true);
            return then.get();
        }
        finally {
            ResourcePackHook.setDisabled(false);
        }
    }

    public static <TEx extends Throwable> void disable(ThrowingRunnable<TEx> then) throws TEx {
        try {
            ResourcePackHook.setDisabled(true);
            then.run();
        } finally {
            ResourcePackHook.setDisabled(false);
        }
    }

    public static final Event<Contains> CONTAINS = EventFactory.createArrayBacked(Contains.class,
            (listeners) -> (type, id, previous, pack) -> {
                for (Contains listener : listeners) {
                    previous = listener.contains(type, id, previous, pack);
                }
                return previous;
            });

    public static final Event<FindResource> FIND_RESOURCE = EventFactory.createArrayBacked(FindResource.class,
            (listeners) -> ((type, namespace, prefix, maxDepth, pathFilter, previous, pack) -> {
                for (FindResource listener : listeners) {
                    previous = listener.findResources(type, namespace, prefix, maxDepth, pathFilter, previous, pack);
                }
                return previous;
            }));

    public static final Event<Open> OPEN = EventFactory.createArrayBacked(Open.class,
            (listeners) -> ((type, id, previous, pack) -> {
                for (Open listener : listeners) {
                    previous = listener.open(type, id, previous, pack);
                }
                return previous;
            }));

    public static final Event<OpenRoot> OPEN_ROOT = EventFactory.createArrayBacked(OpenRoot.class,
            (listeners) -> ((fileName, previous, pack) -> {
                for (OpenRoot listener : listeners) {
                    previous = listener.openRoot(fileName, previous, pack);
                }
                return previous;
            }));

    public interface Contains {
        boolean contains(ResourceType type, Identifier id, boolean previous, ResourcePack pack);
    }

    public interface FindResource {
        Collection<Identifier> findResources(ResourceType type, String namespace, String prefix, int maxDepth, Predicate<String> pathFilter, Collection<Identifier> previous, ResourcePack pack);
    }

    public interface Open {
        InputStream open(ResourceType type, Identifier id, InputStream previous, ResourcePack pack) throws IOException;
    }

    public interface OpenRoot {
        InputStream openRoot(String fileName, InputStream previous, ResourcePack pack) throws IOException;
    }
}
