package io.gitlab.jfronny.libjf.data.manipulation.impl;

import io.gitlab.jfronny.libjf.ResourcePath;
import io.gitlab.jfronny.libjf.data.manipulation.api.UserResourceEvents;
import net.minecraft.resource.ResourcePack;
import net.minecraft.resource.ResourceType;
import net.minecraft.util.Identifier;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.function.Predicate;

@SuppressWarnings("unused")
public class ResourcePackHook {
    private static final Map<Long, Boolean> disabled = new HashMap<>();

    public static void setDisabled(boolean disabled) {
        ResourcePackHook.disabled.put(Thread.currentThread().getId(), disabled);
    }

    public static boolean isDisabled() {
        return disabled.getOrDefault(Thread.currentThread().getId(), false);
    }

    public static boolean hookContains(boolean value, ResourcePack pack, ResourceType type, Identifier id) {
        return isDisabled() ? value : UserResourceEvents.CONTAINS.invoker().contains(type, id, value, pack);
    }
    public static InputStream hookOpen(InputStream value, ResourcePack pack, ResourceType type, Identifier id) throws IOException {
        if (isDisabled()) return value;
        InputStream is = UserResourceEvents.OPEN.invoker().open(type, id, value, pack);
        if (is == null)
            throw new FileNotFoundException(new ResourcePath(type, id).getName() + "CN");
        return is;
    }
    public static InputStream hookOpenEx(IOException ex, ResourcePack pack, ResourceType type, Identifier id) throws IOException {
        if (isDisabled()) throw ex;
        try {
            return hookOpen(null, pack, type, id);
        }
        catch (Throwable t) {
            throw ex;
        }
    }
    public static Collection<Identifier> hookFindResources(Collection<Identifier> value, ResourcePack pack, ResourceType type, String namespace, String prefix, int maxDepth, Predicate<String> pathFilter) {
        return isDisabled() ? value : UserResourceEvents.FIND_RESOURCE.invoker().findResources(type, namespace, prefix, maxDepth, pathFilter, value, pack);
    }
    public static InputStream hookOpenRoot(InputStream value, ResourcePack pack, String fileName) throws IOException {
        if (isDisabled()) return value;
        InputStream is = value;
        is = UserResourceEvents.OPEN_ROOT.invoker().openRoot(fileName, is, pack);
        if (is == null)
            throw new FileNotFoundException(fileName);
        return is;
    }
    public static InputStream hookOpenRootEx(IOException ex, ResourcePack pack, String fileName) throws IOException {
        if (isDisabled()) throw ex;
        try {
            return hookOpenRoot(null, pack, fileName);
        }
        catch (Throwable t) {
            throw ex;
        }
    }
}
