package io.gitlab.jfronny.libjf.unsafe.asm.patch;

import io.gitlab.jfronny.libjf.unsafe.asm.AsmTransformer;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.*;

public class PatchUtil {
    public static String getRemappedInternal(String className) {
        return getRemapped(className.replace('/', '.')).replace('.', '/');
    }

    public static String getRemapped(String className) {
        return AsmTransformer.MAPPING_RESOLVER.mapClassName(AsmTransformer.INTERMEDIARY, className);
    }

    public static void redirectReturn(MethodNode method, String targetClass, String hookClass, String targetMethod, String targetType, String[] extraParamTypes) {
        for (AbstractInsnNode node : method.instructions) {
            if (node.getOpcode() == Opcodes.ARETURN
                    || node.getOpcode() == Opcodes.IRETURN
                    || node.getOpcode() == Opcodes.DRETURN
                    || node.getOpcode() == Opcodes.FRETURN
                    || node.getOpcode() == Opcodes.LRETURN) {
                method.instructions.insertBefore(node, PatchUtil.buildParamPassingInvoker(targetClass, hookClass, targetMethod, targetType, targetType, extraParamTypes));
            }
        }
    }

    public static void redirectExceptions(MethodNode method, String targetClass, String hookClass, String exceptionType, String returnTypeNormal, String targetMethod, String[] extraParamTypes) {
        LabelNode start = new LabelNode();
        LabelNode end = new LabelNode();
        LabelNode handler = new LabelNode();
        method.instructions.insertBefore(method.instructions.getFirst(), start);
        method.instructions.insertBefore(method.instructions.getLast(), end);
        method.instructions.add(handler);
        method.instructions.add(PatchUtil.buildParamPassingInvoker(targetClass,
                hookClass,
                targetMethod,
                "L" + getRemappedInternal(exceptionType) + ";",
                "L" + getRemappedInternal(returnTypeNormal) + ";",
                extraParamTypes));
        method.instructions.add(new InsnNode(Opcodes.ARETURN));
        method.tryCatchBlocks.add(new TryCatchBlockNode(start, end, handler, exceptionType));
    }

    public static InsnList buildParamPassingInvoker(String targetClass, String hookClass, String targetMethod, String inType, String outType, String[] extraParamTypes) {
        InsnList instructions = new InsnList();
        instructions.add(new VarInsnNode(Opcodes.ALOAD, 0));
        StringBuilder descriptor = new StringBuilder("(").append(inType);
        descriptor.append('L').append(getRemappedInternal(targetClass)).append(';');
        for (int i = 0; i < extraParamTypes.length; i++) {
            instructions.add(new VarInsnNode(switch (extraParamTypes[i]) {
                case "I" -> Opcodes.ILOAD;
                case "D" -> Opcodes.DLOAD;
                case "F" -> Opcodes.FLOAD;
                case "L" -> Opcodes.LLOAD;
                default -> Opcodes.ALOAD;
            }, i + 1));
            if (extraParamTypes[i].length() == 1)
                descriptor.append(extraParamTypes[i]);
            else
                descriptor.append('L').append(PatchUtil.getRemappedInternal(extraParamTypes[i])).append(';');
        }
        descriptor.append(")").append(outType);
        instructions.add(new MethodInsnNode(Opcodes.INVOKESTATIC, hookClass, targetMethod, descriptor.toString()));
        return instructions;
    }
}
