package io.gitlab.jfronny.libjf.unsafe.asm.patch.modification;

import io.gitlab.jfronny.libjf.LibJf;
import io.gitlab.jfronny.libjf.unsafe.asm.AsmTransformer;
import io.gitlab.jfronny.libjf.unsafe.asm.patch.MethodPatch;
import io.gitlab.jfronny.libjf.unsafe.asm.patch.Patch;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

public class MethodModificationPatch implements Patch {
    private final Map<String, MethodPatch> patches = new LinkedHashMap<>();

    public MethodModificationPatch(String targetMethodOwner, Set<MethodDescriptorPatch> patches) {
        for (MethodDescriptorPatch patch : patches) {
            this.patches.put(
                    AsmTransformer.MAPPING_RESOLVER.mapMethodName(AsmTransformer.INTERMEDIARY, targetMethodOwner, patch.methodName, patch.methodDescriptor),
                    patch.patch
            );
        }
        for (String s : this.patches.keySet()) {
            if (AsmTransformer.INSTANCE.debugLogsEnabled()) LibJf.LOGGER.info("Registered patch for " + s);
        }
    }

    @Override
    public void apply(ClassNode klazz) {
        for (MethodNode method : klazz.methods) {
            if (patches.containsKey(method.name)) {
                if (AsmTransformer.INSTANCE.debugLogsEnabled()) LibJf.LOGGER.info("Patching " + method.name);
                patches.get(method.name).apply(method);
            }
        }
    }

    public static record MethodDescriptorPatch(String methodName, String methodDescriptor, MethodPatch patch) {

    }
}
